@@ -1,4 +1,7 @@
-from glue_vispy_viewers.extern.vispy.visuals.transforms import ChainTransform, NullTransform, MatrixTransform, STTransform
+from __future__ import absolute_import, division, print_function
+
+from glue_vispy_viewers.extern.vispy.visuals.transforms import (ChainTransform, NullTransform,
+                                                                MatrixTransform, STTransform)
 from glue_vispy_viewers.extern.vispy.visuals.transforms.base_transform import InverseTransform
 
 
